﻿""" Power Glove WorldViz Vizard demo v1.1.  Kevin Mellott. 2014
Relies on the PGLink2 adapter board from: 
http://www.mellottsvrpage.com/index.php/power-glove-linkbox-usb/

Make sure you set the COM port 'COM11' to the one your PGLink2 board is using.
Don't change baudrate, bytesize, stopbits, and parity unless you've 
modified the PGLink2 firmware to match.
"""

import viz
import vizact
import hand
import serial
import time
import vizcam

#Load the most recent custom handPG.py file
import handPG
reload(handPG)

#Setup the main viz environment
viz.setMultiSample(4)
viz.fov(60)
viz.go(viz.FULLSCREEN)
#Grab the main view. 
view = viz.MainView
viz.MainView.setPosition(1,1,0) 
#viz.cam.setHandler(vizcam.KeyboardCamera(forward='w',backward='s', left='a',right='d',turnRight='e',turnLeft='q'))
#viz.mouse.setVisible(viz.OFF)

#Replace this with whatever WorldViz environment you want
piazza = viz.addChild('piazza.osgb')
#ball = viz.addChild('soccerball.ive', pos = [0,1,3])

#Setup the power glove's serial port assuming PGLink2 adapter board.
ser = serial.Serial('COM11', 115200, bytesize=8, stopbits=1, parity=serial.PARITY_EVEN) 
if ser.isOpen():
    ser.close()
ser.open()
if ser.isOpen():
     print ser.portstr    

#Power Glove Reset and Hi-Res commands
ser.write("RESET\r".encode()) # Send Reset to PG
time.sleep(0.1)
ser.write("HI_RES\r".encode()) # Set to Hi-Res mode
time.sleep(0.1)
ser.write("HI_RES\r".encode()) # Sometimes PG needs a second go
time.sleep(0.1)
ser.flushInput()

#Set Power Glove PGLink2 Filter options
ser.write("XHYST 2\r".encode()) 
time.sleep(0.1)
ser.write("YHYST 2\r".encode()) 
time.sleep(0.1)
ser.write("XACC 8\r".encode()) 
time.sleep(0.1)
ser.write("YACC 8\r".encode()) 
time.sleep(0.1)
ser.write("XXTEND 1\r".encode()) 
time.sleep(0.1)
ser.write("YXTEND 1\r".encode()) 
time.sleep(0.1)
ser.flushInput()

#Create a hand object from the data glove
glove=handPG.add(None, abduct = True, wrist = False)
glove.rightHand()

#Place the hand in front of the user
glove.setEuler([0,-45,0])

def getPG():
     pgdata = ser.readline() # read a 'n' terminated line 
     #print pgdata
     if pgdata:
          #Parse the incoming 12byte ASCII format data
          sp=pgdata.split(",")
          
          #Grab the x,y,z and rotation data
          x=float(sp[0].split()[1])/255.0
          y=float(sp[1].split()[1])/255.0
          z=float(sp[2].split()[1])/255.0
          rot=float(sp[3].split()[1])*360.0/11.0
          pos=view.getPosition();
          #print pos
          print x,y,z,rot
          
          #Grab and parse the finger data.
          fingers=sp[4].split()[1]
          finger_array=list(fingers)
          thumb=float(finger_array[0])
          index=90.0*float(finger_array[1])/3.0
          middle=90.0*float(finger_array[2])/3.0
          ring=90.0*float(finger_array[3])/3.0
          little=ring
          if thumb>1.0:
               thumb=90.0
          else:
               thumb=0.0
          
          #Update the glove model 
          glove.setPosition([0.2+x+pos[0],-0.15+y+pos[1],0.5-z+pos[2]])
          glove.setEuler([0,-45,-rot])
          glove.setFingers(thumb,index,middle,ring,little)

def onExit():
     ser.close() # close port

viz.callback(viz.EXIT_EVENT,onExit)
vizact.ontimer(0, getPG)
